@@ -18,11 +18,10 @@
 #' }
 #' @seealso 
 #' * [set_bvar()] to specify the hyperparameters of Minnesota prior.
-#' * [bvar_adding_dummy] for dummy observations definition.
 #' @references 
 #' Bańbura, M., Giannone, D., & Reichlin, L. (2010). *Large Bayesian vector auto regressions*. Journal of Applied Econometrics, 25(1).
 #' 
-#' Karlsson, S. (2013). *Chapter 15 Forecasting with Bayesian Vector Autoregression*. Handbook of Economic Forecasting, 2, 791–897.
+#' Karlsson, S. (2013). *Chapter 15 Forecasting with Bayesian Vector Autoregression*. Handbook of Economic Forecasting, 2, 791-897.
 #' 
 #' Litterman, R. B. (1986). *Forecasting with Bayesian Vector Autoregressions: Five Years of Experience*. Journal of Business & Economic Statistics, 4(1), 25.
 #' @examples 
@@ -74,23 +73,25 @@ sim_mncoef <- function(p, bayes_spec = set_bvar(), full = TRUE) {
   iw_shape <- prior$prior_shape
   # random---------------------------
   if (full) {
-    res <- sim_mniw(
+    res <- sim_mniw_export(
       1,
       mn_mean, # mean of MN
       solve(mn_prec), # scale of MN = inverse of precision
       iw_scale, # scale of IW
-      iw_shape # shape of IW
-    )
+      iw_shape, # shape of IW
+      FALSE
+    )[[1]]
     res <- list(
-      coefficients = res$mn,
-      covmat = res$iw
+      coefficients = res[[1]],
+      covmat = res[[2]]
     )
   } else {
     sig <- diag(sigma^2)
     res <- sim_matgaussian(
       mn_mean,
-      solve(mn_prec),
-      sig
+      mn_prec,
+      sig,
+      TRUE
     )
     res <- list(
       coefficients = res,
@@ -113,13 +114,12 @@ sim_mncoef <- function(p, bayes_spec = set_bvar(), full = TRUE) {
 #' @seealso 
 #' * [set_bvhar()] to specify the hyperparameters of VAR-type Minnesota prior.
 #' * [set_weight_bvhar()] to specify the hyperparameters of HAR-type Minnesota prior.
-#' * [bvar_adding_dummy] for dummy observations definition.
 #' @return List with the following component.
 #' \describe{
 #'   \item{coefficients}{BVHAR coefficient (MN)}
 #'   \item{covmat}{BVHAR variance (IW or diagonal matrix of `sigma` of `bayes_spec`)}
 #' }
-#' @references Kim, Y. G., and Baek, C. (n.d.). *Bayesian vector heterogeneous autoregressive modeling*. submitted.
+#' @references Kim, Y. G., and Baek, C. (2024). *Bayesian vector heterogeneous autoregressive modeling*. Journal of Statistical Computation and Simulation, 94(6), 1139-1157.
 #' @examples 
 #' # Generate (Phi, Sigma)
 #' # BVHAR-S
@@ -195,23 +195,25 @@ sim_mnvhar_coef <- function(bayes_spec = set_bvhar(), full = TRUE) {
   iw_shape <- prior$prior_shape
   # random---------------------------
   if (full) {
-    res <- sim_mniw(
+    res <- sim_mniw_export(
       1,
       mn_mean, # mean of MN
       solve(mn_prec), # scale of MN = inverse of precision
       iw_scale, # scale of IW
-      iw_shape # shape of IW
-    )
+      iw_shape, # shape of IW
+      FALSE
+    )[[1]]
     res <- list(
-      coefficients = res$mn,
-      covmat = res$iw
+      coefficients = res[[1]],
+      covmat = res[[2]]
     )
   } else {
     sig <- diag(sigma^2)
     res <- sim_matgaussian(
       mn_mean,
-      solve(mn_prec),
-      sig
+      mn_prec,
+      sig,
+      TRUE
     )
     res <- list(
       coefficients = res,
@@ -223,7 +225,7 @@ sim_mnvhar_coef <- function(bayes_spec = set_bvhar(), full = TRUE) {
 
 #' Generate SSVS Parameters
 #' 
-#' This function generates parameters of VAR with SSVS prior.
+#' `r lifecycle::badge("deprecated")` This function generates parameters of VAR with SSVS prior.
 #' 
 #' @param bayes_spec A SSVS model specification by [set_ssvs()].
 #' @param p VAR lag
@@ -231,7 +233,7 @@ sim_mnvhar_coef <- function(bayes_spec = set_bvhar(), full = TRUE) {
 #' @param include_mean Add constant term (Default: `TRUE`) or not (`FALSE`)
 #' @param minnesota Only use off-diagonal terms of each coefficient matrices for restriction.
 #' In `sim_ssvs_var()` function, use `TRUE` or `FALSE` (default).
-#' In `sim_ssvs_vhar()` function, `"no"` (default), `"short"` type, or `"longrun"` type.
+#' In `sim_ssvs_vhar()` function, `no` (default), `short` type, or `longrun` type.
 #' @param mn_prob Probability for own-lags.
 #' @param method Method to compute \eqn{\Sigma^{1/2}}.
 #' @section VAR(p) with SSVS prior:
@@ -243,13 +245,13 @@ sim_mnvhar_coef <- function(bayes_spec = set_bvhar(), full = TRUE) {
 #' \deqn{(\psi_{ii}^2)}
 #' @return List including coefficients.
 #' @references 
-#' George, E. I., & McCulloch, R. E. (1993). *Variable Selection via Gibbs Sampling*. Journal of the American Statistical Association, 88(423), 881–889.
+#' George, E. I., & McCulloch, R. E. (1993). *Variable Selection via Gibbs Sampling*. Journal of the American Statistical Association, 88(423), 881-889.
 #' 
-#' George, E. I., Sun, D., & Ni, S. (2008). *Bayesian stochastic search for VAR model restrictions*. Journal of Econometrics, 142(1), 553–580.
+#' George, E. I., Sun, D., & Ni, S. (2008). *Bayesian stochastic search for VAR model restrictions*. Journal of Econometrics, 142(1), 553-580.
 #' 
 #' Ghosh, S., Khare, K., & Michailidis, G. (2018). *High-Dimensional Posterior Consistency in Bayesian Vector Autoregressive Models*. Journal of the American Statistical Association, 114(526).
 #' 
-#' Koop, G., & Korobilis, D. (2009). *Bayesian Multivariate Time Series Methods for Empirical Macroeconomics*. Foundations and Trends® in Econometrics, 3(4), 267–358.
+#' Koop, G., & Korobilis, D. (2009). *Bayesian Multivariate Time Series Methods for Empirical Macroeconomics*. Foundations and Trends® in Econometrics, 3(4), 267-358.
 #' @importFrom stats rbinom rnorm rgamma
 #' @export
 sim_ssvs_var <- function(bayes_spec,
@@ -259,6 +261,7 @@ sim_ssvs_var <- function(bayes_spec,
                          minnesota = FALSE,
                          mn_prob = 1,
                          method = c("eigen", "chol")) {
+  deprecate_warn("2.0.1", details = "The package uses different SSVS specification.")
   if (!is.ssvsinput(bayes_spec)) {
     stop("Provide 'ssvsinput' for 'bayes_spec'.")
   }
@@ -378,6 +381,7 @@ sim_ssvs_vhar <- function(bayes_spec,
                           minnesota = c("no", "short", "longrun"),
                           mn_prob = 1,
                           method = c("eigen", "chol")) {
+  deprecate_warn("2.0.1", details = "The package uses different SSVS specification.")
   if (!is.ssvsinput(bayes_spec)) {
     stop("Provide 'ssvsinput' for 'bayes_spec'.")
   }
@@ -492,14 +496,14 @@ sim_ssvs_vhar <- function(bayes_spec,
 
 #' Generate Horseshoe Parameters
 #'
-#' This function generates parameters of VAR with Horseshoe prior.
+#' `r lifecycle::badge("deprecated")` This function generates parameters of VAR with Horseshoe prior.
 #' 
 #' @param p VAR lag
 #' @param dim_data Specify the dimension of the data if hyperparameters of `bayes_spec` have constant values.
 #' @param include_mean Add constant term (Default: `TRUE`) or not (`FALSE`)
 #' @param minnesota Only use off-diagonal terms of each coefficient matrices for restriction.
 #' In `sim_horseshoe_var()` function, use `TRUE` or `FALSE` (default).
-#' In `sim_horseshoe_vhar()` function, `"no"` (default), `"short"` type, or `"longrun"` type.
+#' In `sim_horseshoe_vhar()` function, `no` (default), `short` type, or `longrun` type.
 #' @param method Method to compute \eqn{\Sigma^{1/2}}.
 #' @importFrom stats rbinom rnorm rgamma
 #' @export
@@ -508,6 +512,7 @@ sim_horseshoe_var <- function(p,
                               include_mean = TRUE,
                               minnesota = FALSE,
                               method = c("eigen", "chol")) {
+  deprecate_warn("2.0.1", details = "The package uses different Horseshoe specification.")
   dim_design <- ifelse(include_mean, dim_data * p + 1, dim_data * p)
   num_coef <- dim_data * dim_design
   num_alpha <- dim_data^2 * p
@@ -573,6 +578,7 @@ sim_horseshoe_vhar <- function(har = c(5, 22),
                                include_mean = TRUE,
                                minnesota = c("no", "short", "longrun"),
                                method = c("eigen", "chol")) {
+  deprecate_warn("2.0.1", details = "The package uses different Horseshoe specification.")
   dim_har <- ifelse(include_mean, 3 * dim_data + 1, 3 * dim_data)
   num_coef <- dim_data * dim_har
   num_phi <- 3 * dim_data^2
